/*
  Copyright (C) 2002 Grzegorz Godlewski <ggodlewski@wi.ps.pl>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <php.h>
#include <zend_globals.h>
#include <config.h>

void xmlerror(void *ctx,char *msg,...)
{
  va_list ap;
  char buf[1024];
  
  zval **errwar;
  zval *newelement;

  errwar=(zval**) &(((xmlValidCtxt*)ctx)->userData);
  
  va_start(ap,msg);
  vsnprintf(buf,1024,msg,ap);
  va_end(ap);

  MAKE_STD_ZVAL(newelement);
  ZVAL_STRING(newelement,buf,1);

  if (zend_hash_next_index_insert(HASH_OF(*errwar),(void *)&newelement,sizeof(zval *),NULL)==FAILURE)
  {

    zend_error(E_ERROR,"PXV: Inserting error into array failed");
  }
};

void xmlwarning(void *ctx,char *msg,...)
{
  va_list ap;
  char buf[1024];

  zval **errwar;
  zval *newelement;
  
  errwar=(zval**) &(((xmlValidCtxt*)ctx)->userData);
  
  va_start(ap,msg);
  vsnprintf(buf,1024,msg,ap);
  va_end(ap);
  
  MAKE_STD_ZVAL(newelement);
  ZVAL_STRING(newelement,buf,1);

  if (zend_hash_next_index_insert(errwar[1]->value.ht,(void *)&newelement,sizeof(zval *),NULL)==FAILURE)
  {
    zend_error(E_ERROR,"PXV: Inserting warning into array failed");
  }
};

ZEND_FUNCTION(pxvvalidatedtd);

zend_function_entry pxv_functions[]=
{
  ZEND_FE(pxvvalidatedtd,NULL)
  {NULL,NULL,NULL}
};

zend_module_entry pxv_entry=
{
  STANDARD_MODULE_HEADER,
  "pxv",
  pxv_functions,
  NULL,NULL,NULL,
  NULL,NULL,
  NO_VERSION_YET,
  STANDARD_MODULE_PROPERTIES
};

#if COMPILE_DL_PXV
DLEXPORT zend_module_entry *get_module(void) {return(&pxv_entry);}
#endif

ZEND_FUNCTION(pxvvalidatedtd)
{
  zval **par,**par2,**par3,**par4;
  FILE *fp;
  int rm;
  xmlDocPtr doc;
  xmlDtdPtr dtd;
  xmlValidCtxt cvp;
  zval *errwar[2];
  int args;


  
  zval *el;




  MAKE_STD_ZVAL(errwar[0]);
  MAKE_STD_ZVAL(errwar[1]);
  
  if (array_init(errwar[0])!=SUCCESS)
  {
    zend_error(E_ERROR,"PXV: creating error array failed");
  }

  if (array_init(errwar[1])!=SUCCESS)
  {
    zend_error(E_ERROR,"PXV: creating warning array failed");
  }

  if ((args=ZEND_NUM_ARGS())<2)
  {
    WRONG_PARAM_COUNT;
  }

  switch (args)
  {
    case 2:
      if (zend_get_parameters_ex(2,&par,&par2)!=SUCCESS)
      {
        WRONG_PARAM_COUNT;
      }
      break;
    case 3:
      if (zend_get_parameters_ex(3,&par,&par2,&par3)!=SUCCESS)
      {
        WRONG_PARAM_COUNT;
      }
      if (!(*par3)->is_ref)
      {
        zend_error(E_ERROR,"PXV: 3rd parameter isn't passed by reference");
      }
      break;
    case 4:
      if (zend_get_parameters_ex(4,&par,&par2,&par3,&par4)!=SUCCESS)
      {
        WRONG_PARAM_COUNT;
      }
      if (!(*par4)->is_ref)
      {
        zend_error(E_ERROR,"PXV: 4rd parameter isn't passed by reference");
      }
      break;
    default:
      WRONG_PARAM_COUNT;
  }
  
  if ((*par)->type!=IS_STRING)
  {
    zend_error(E_ERROR,"PXV: 1rd parameter isn't a string");
  };

  if ((*par2)->type!=IS_STRING)
  {
    zend_error(E_ERROR,"PXV: 2nd parameter isn't a string");
  };

  doc=xmlParseMemory((*par)->value.str.val,(*par)->value.str.len);
  if (doc==NULL)
  {
    zend_error(E_ERROR,"PXV: preparing document for validator failed");
    RETURN_BOOL(0);
  }

  dtd = xmlParseDTD(NULL, (const xmlChar *)(*par2)->value.str.val);
  if (dtd == NULL)
  {
    zend_error(E_ERROR,"PXV: preparing DTD for validator failed");
    RETURN_BOOL(0);
  }
  
  cvp.userData = (void *) errwar;
  cvp.error    = (xmlValidityErrorFunc) xmlerror;
  cvp.warning  = (xmlValidityWarningFunc) xmlwarning;

  if (!xmlValidateDtd(&cvp, doc, dtd))
  {
//    xmlGenericError(xmlGenericErrorContext,"Document does not validate against\n");
    if (args>=3)memcpy((*par3),errwar[0],sizeof(zval *));
    if (args>=4)memcpy((*par4),errwar[1],sizeof(zval *));
    xmlFreeDoc(doc);
    xmlFreeDtd(dtd);
    RETURN_BOOL(0);
  } else
  {
    if (args>=3)memcpy((*par3),errwar[0],sizeof(zval *));
    if (args>=4)memcpy((*par4),errwar[1],sizeof(zval *));
    xmlFreeDoc(doc);
    xmlFreeDtd(dtd);
    RETURN_BOOL(1);
  }
};
